#!/usr/bin/perl -wall
use Fcntl;
use File::Copy;
use File::Find;

#hash of bad files to start with
@badToSearch= ( "WikiWikiWeb.html", "PageGroupTest.html" );

#already looked through - a hash for easy searching
%badFiles=();

#repeat until badToSearch is empty
while (scalar(@badToSearch)) {
  #get the first file from the list
  $file=shift(@badToSearch);
  print "\nchecking in file $file...";
  #open the file using sysopen to avoid problems with odd file names
  sysopen(IN, $file, O_RDONLY) or die "Couldn't open $file for reading: $!\n";
  #search the file for links.  Check %badFiles to see if the link has already been logged
  while (<IN>) {
	chomp;
	$line=$_;
	while ($line=~s/a href="(.*?)"(.*)/$2/) {
	  $check=$1;

	  #we don't want to include *_BackLinks* pages ... this page could have been referenced
	  #by a page we want to keep.  We'll take care of BackLink pages as part of the cleanup for
	  #the original page
	  #same for HomePage
	  next if ($check=~/BackLink|HomePage/);

	  #make sure this link hasn't already been tested, or about to be, and that it's a file that exists in this dir
	  if ((! exists($badFiles{$check})) && (! grep $_ eq $check, @badToSearch ) && (-f $check)) {
		print "...adding file $check to search";
		push(@badToSearch, $check);
	  }
	}
  }
  close(IN);
  #add this file to %badFiles
  $badFiles{$file}="1";
}

print "Bad files are:";
foreach $file (keys(%badFiles)) {
  print "$file";
}
print "\n\n";

#process the bad files

#sub to process files
sub wanted {
  return if (!($File::Find::name=~m/html/) || !(-f $File::Find::name));
  print ">>>Adding $File::Find::name to list of files to be processed";
  push(@filesToProcess, $File::Find::name);
}


#iterate over keys(badfiles)
#remove all these files so we don't spend time processing them
foreach $badfile (keys(%badFiles)) {
  #create vars for badfile and badfile_BackLinks.html
  $badfileBL=$badfile;
  $badfileBL=~s/\.html/_BackLinks.html/;
  print ">>>Unlinking $badfile and $badfileBL";
  unlink($badfile);
  unlink($badfileBL);
}

@filesToProcess=();
#iterate over all html files
find(\&wanted, ".");


foreach $badfile (keys(%badFiles)) {
  #create vars for badfile and badfile_BackLinks.html
  $badfileBL=$badfile;
  $badfileBL=~s/\.html/_BackLinks.html/;

  print ">>>Removing all refs to $badfile and $badfileBL";

  #iterate all files to be processed
  foreach $file (@filesToProcess) {
	print "\t>>>Processing file $file";
	#open the file using sysopen to avoid problems with odd file names
	sysopen(IN, $file, O_RDONLY) or die "Couldn't open $file for reading: $!\n";
	#read file in to array
	@contents=<IN>;
	close(IN);
	#iterate over array
	$changed=0;
	@newContents=();
	foreach $line (@contents) {
	  $oldline=$line;
	  #perl cookbook to the rescue...
	  #replace all links to badfile and badfile_BackLinks
	  1 while ($line=~s/(.*?)<a href="($badfile|$badfileBL).*?>(.*?)<\/a>(.*)/$1$3$4/);
	  if ($oldline ne $line) {
		$changed=1;
		print "\t\t>>>Line Changed:\n$oldline\n---->\n$line";
	  }
	  push(@newContents, $line);
	}
	#write array to file- only if modified
	if ($changed) {
	  $origFile=$file.".orig";
	  #only copy it to backup if backup doesn't already exist
	  if ( ! -f $origFile) {
	  	copy($file, $origFile);
	  }
	  print "\t\t>>>Writing changes to $nf";
	  sysopen(OUT, $file, O_WRONLY|O_CREAT) or die "Couldn't open $file for writing: $!\n";
	  print OUT @newContents;
	  close(OUT);
	}
  }
}
